@@ -15,7 +15,7 @@ use List::MoreUtils                 qw(any firstval);
 
 use Mojolicious::Plugin::Vparam::Address;
 
-our $VERSION = '1.4';
+our $VERSION = '1.5';
 
 =encoding utf-8
 
@@ -205,7 +205,7 @@ Set new type $name if defined %opts. Else return type $name definition.
         }
     );
 
-=head2 vfilter $name, $sub
+=head2 vfilter $name, &sub
 
 Set new filter $name if defined %opts. Else return filter $name definition.
 
@@ -229,15 +229,21 @@ simply by using an expression instead.
 
 =item I<REGEXP>
 
-Apply as B<regexp> filter. No type verification, just match.
+Apply as L</regexp> filter. No type verification, just match.
+
+    $self->vparam(myparam => qr{^(abc|cde)$});
 
 =item I<CODE> $mojo, $value
 
-Apply as B<post> function. You need manual verify and set error.
+Apply as L</post> function. You need manual verify and set error.
+
+    $self->vparam(myparam => sub { $_[1] && $_[1] eq 'good' ? 1 : 0 });
 
 =item I<ARRAY>
 
-Apply as B<in> filter. No type verification, just match.
+Apply as L</in> filter. No type verification, just match.
+
+    $self->vparam(myparam => [qw(abc cde)]);
 
 =back
 
@@ -489,7 +495,30 @@ Timestamp.
 
 =item *
 
-Relative in minutes. For example: +15 or -6 from now.
+Relative from now in format C<[+-] DD HH:MM:SS>. First sign required.
+
+=over
+
+=item *
+
+Minutes by default. Example: C<+15> or C<-6>.
+
+=item *
+
+Minutes and seconds. Example: C<+15:44>.
+
+=item *
+
+Hours. Example: C<+3:15:44>.
+
+=item *
+
+Days. Example: C<+8 3:15:44>.
+
+=back
+
+Values are given in arbitrary range.
+For example you can add 400 minutes and 300 seconds: C<+400:300>.
 
 =item *
 
@@ -884,7 +913,7 @@ sub _range($$$) {
 Check parameter to be match for regexp
 
     # Error if myparam not equal "abc" or "cde"
-    $self->vparam(myparam => 'int', regexp => qr{^(abc|cde)$});
+    $self->vparam(myparam => 'str', regexp => qr{^(abc|cde)$});
 
 =cut
 
@@ -983,10 +1012,21 @@ sub _parse_date($;$) {
 
     if( $str =~ m{^\d+$} ) {
         $dt = DateTime->from_epoch( epoch => int $str, time_zone => 'local' );
-    } elsif( $str =~ m{^[\+\-]\d+$} ) {
-        my $minutes = int $str;
+    } elsif( $str =~ m{^[+-]} ) {
+        my @relative = $str =~ m{
+            ^([+-])             # sign
+            \s*
+            (?:(\d+)\s+)?       # days
+            (?:(\d+):)??        # hours
+            (\d+)               # minutes
+            (?::(\d+))?         # seconds
+        $}x;
         $dt = DateTime->now(time_zone => 'local');
-        $dt->add(minutes => $minutes);
+        my $sub = $relative[0] eq '+' ? 'add' : 'subtract';
+        $dt->$sub(days      => int $relative[1])    if defined $relative[1];
+        $dt->$sub(hours     => int $relative[2])    if defined $relative[2];
+        $dt->$sub(minutes   => int $relative[3])    if defined $relative[3];
+        $dt->$sub(seconds   => int $relative[4])    if defined $relative[4];
     } else {
         # RU format
         $str =~ s{^(\d{1,2})\.(\d{1,2})\.(\d{4})(.*)$}{$3-$2-$1$4};
@@ -995,10 +1035,11 @@ sub _parse_date($;$) {
             if $str =~ m{^\s*\d{2}:};
 
         $dt = eval { DateTime::Format::DateParse->parse_datetime( $str ); };
-        return undef if !$dt;
         return undef if $@;
     }
 
+    return undef unless $dt;
+
     # Always local timezone
     $tz //= DateTime->now(time_zone => 'local')->strftime('%z');
     $dt->set_time_zone( $tz );
@@ -1358,7 +1399,7 @@ sub register {
 
             # Get value
             my @input;
-            if( version->new($Mojolicious::VERSION) < version->new(5.28) ) {
+            if( version->new($Mojolicious::VERSION) < version->new(5.48) ) {
                 @input = $self->param( $name );
             } else {
                 @input = @{ $self->every_param( $name ) };
